package jadex.bdi.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import jadex.model.annotation.NameValue;


/**
 *  Redefines jadex.micro.annotation.Configuration
 *  as annotations do not allow inheritance.
 *  
 *  Configuration annotation.
 */
//@Target({ElementType.TYPE, ElementType.ANNOTATION_TYPE})
@Target(ElementType.ANNOTATION_TYPE)
@Retention(RetentionPolicy.RUNTIME)
public @interface BDIConfiguration
{	
	//-------- methods of Configuration --------
	
	/**
	 *  The configuration name.
	 */
	public String name();
	
	/**
	 *  The arguments.
	 */
	public NameValue[] arguments() default {};
	
	/**
	 *  The results.
	 */
	public NameValue[] results() default {};

//	/**
//	 *  The components.
//	 */
//	public Component[] components() default {};
//	
//	/**
//	 *  The provided service implementations.
//	 */
//	public ProvidedService[] providedservices() default {};
//	
//	/**
//	 *  The provided service implementations.
//	 */
//	public RequiredService[] requiredservices() default {};
	
	/**
	 *  The synchronous flag.
	 */
	public boolean synchronous() default false;
	
	/**
	 *  The persistable flag.
	 */
	public boolean persistable() default false;
	
	/**
	 *  The suspend flag.
	 */
	public boolean suspend() default false;
	
	//-------- new bdi methods --------
	
	/**
	 *  The initial beliefs.
	 */
	public NameValue[] initialbeliefs() default {};

	/**
	 *  The initial goals.
	 */
	public NameValue[] initialgoals() default {};

	/**
	 *  The initial plans.
	 */
	public NameValue[] initialplans() default {};

	/**
	 *  The end beliefs.
	 */
	public NameValue[] endbeliefs() default {};

	/**
	 *  The end goals.
	 */
	public NameValue[] endgoals() default {};

	/**
	 *  The end plans.
	 */
	public NameValue[] endplans() default {};
}




